#!/usr/bin/env node
'use strict'

const program = require('commander')
const _axios = require('axios')
const prompt = require('prompt')
const fs = require('fs')
const util = require('util')
// BUILD NEW
// npm install -g
// npm link
// npm publish
const makeSocket = require('socket.io-client')
const ss = require('socket.io-stream')
const path = require('path')
const env = require('dotenv')

const exec = require('child_process').exec;
const run = call => {
  return new Promise( (resolve,reject) => {
    exec(call, (error, stdout, stderr) => {
      if(stderr){
        //console.log(stderr)
      }
      if (error) {
        //console.log(error)
      }
      resolve(stdout)
    })
  })
}
var isWin = /^win/.test(process.platform)
let mkdir = folder => 'mkdir '+folder
if(isWin){
  mkdir = folder => 'mkdir "'+folder+'"'
}

const uploadFile = (socket, folder, key) => filePath => {
  return new Promise((resolve, reject) => {
    const stream = ss.createStream()
    const file = fixPath(folder)(filePath)
    ss(socket).emit('file', stream, {name: file,key: key})
    const stream2 = fs.createReadStream(filePath)
    let size = 0
    let totalSize = fs.statSync(filePath).size
    let isDone = true

    stream2.on('data', (chunk) => {
      size += chunk.length
      const status = Math.floor(size / totalSize * 100)
      isDone = status === 100
      if(!isDone){
        console.log(file + ': '+status+'%')
      }
    })

    const timer = setInterval( () => {
      if(isDone){
        console.log(file + ': 100%')
        clearInterval(timer)
        resolve()
      }
    }, 2000)

    stream2.pipe(stream)
  })
}

const ignoreList = [
  '.bin',
  'node_modules',
  '.git'
]

const mapFilesOnly = folder => (result,fileOrFolder) => {
  if(ignoreList.indexOf(fileOrFolder) > -1){
    return result
  }
  return result.concat(
    fs.statSync(folder+'/'+fileOrFolder).isDirectory()
      ? getFiles(folder+'/'+fileOrFolder)
      : [ folder+ '/' + fileOrFolder ]
  )
}

const fixPath = folder => file =>
  file.replace(folder+'/','')

const getFiles = folder =>
  fs.readdirSync(folder)
    .reduce(mapFilesOnly(folder),[])

const uploadFiles = folder => {
  const config = env.config({ path: folder+'/.env' }).parsed
  if( ! config.online_server || ! config.online_key ){
    console.log('no server or key');
    process.exit()
  }
  const socket = makeSocket(config.online_server)
  socket.on('connect', () => {
    Promise.all(
      getFiles(folder).map(uploadFile(socket,folder,config.online_key.replace('#','')))
    ).then( _ => {
      socket.emit('done',{key: config.online_key.replace('#','')})
      console.log('file moved!')
    })
    .catch(console.log)
  })
  socket.on('disconnect', () => {
    console.log('error socket disconnected')
    process.exit()
  })
  socket.on('msg', msg => {
    console.log(msg)
    if(msg == 'done'){
      process.exit()
    }
  })
  socket.on('err', err => {
    console.log(err)
    process.exit()
  })
}

prompt.start()

const envFileUrl =
  'https://raw.githubusercontent.com/js-web-framework/js-web/master/template-files/.env_template'

const indexFileUrl =
  'https://raw.githubusercontent.com/js-web-framework/js-web/master/template-files/index.html'

const appFileUrl =
  'https://raw.githubusercontent.com/js-web-framework/js-web/master/template-files/app.js'

const appSimpleFileUrl =
  'https://raw.githubusercontent.com/js-web-framework/js-web/master/template-files/app-simple.js'


const appBabelFileUrl =
  'https://raw.githubusercontent.com/js-web-framework/js-web/master/template-files/app-babel.js'

const appBabelTemplateFileUrl =
  'https://raw.githubusercontent.com/js-web-framework/js-web/master/template-files/app-babel-template.js'

const dbFileUrl =
  'https://raw.githubusercontent.com/js-web-framework/js-web/master/template-files/db.js'

const runFileUrl =
  'https://raw.githubusercontent.com/js-web-framework/js-web/master/template-files/run.js'

const esLintUrl =
  'https://raw.githubusercontent.com/js-web-framework/js-web/master/template-files/.eslintrc.js'


function downloadFile(url,name){
  _axios.get(url).then(r => {
    const data = r.data
    const file = process.cwd()+'/'+name
    fs.writeFileSync(file,data)
  })
}

const getBabelText = _ => `"babel": {
    "plugins": ["transform-es2015-modules-commonjs"]
  },
  `

const getJsonFile = _ => fs.readFileSync(process.cwd()+'/'+'package.json','utf-8')
const licenseIndex = file => file.indexOf('"license"')
const insertByIndex = (context,index,text) =>
  context.slice(0,index) + text + context.slice(index,context.length)

const writeJsonFile = content => fs.writeFileSync(process.cwd()+'/'+'package.json',content)
const appendBabel = (file) =>
  insertByIndex(file,licenseIndex(file),getBabelText())

const addTestCommand = () =>
  writeJsonFile(
    getJsonFile()
      .replace('"echo \\"Error: no test specified\\" && exit 1"', '"./node_modules/.bin/jest"')
  )

const appendTestToProject = async () => {
    addTestCommand()
    await run('npm install jest --save')
 }

const ifArgPromise = (args,arg,func) => {
  if (args.indexOf(arg) > -1)
    return func()

  return Promise.resolve()
}
const ifArg = (args,arg,func) => {
  if (args.indexOf(arg) > -1)
    return func()
}

const babelAlias = [
  'es6',
  'module',
  'modules'
]

const transformBabel = args =>
  args.map(a => babelAlias.indexOf(a) > -1 ? 'babel' : a)

program
  .command('init')
  .action(function(...args){
    const onlyArgs = transformBabel(args.filter(a => typeof a === 'string'))

    if(onlyArgs.indexOf('all') > -1){
      onlyArgs.push('mysql')
      onlyArgs.push('eslint')
      onlyArgs.push('template')
    }

    if(onlyArgs.indexOf('babel') > -1){
      if(!fs.existsSync(process.cwd()+'/'+'package.json')){
        console.log('no package.json')
        return
      }
      writeJsonFile(appendBabel(getJsonFile()))
    }

    ifArgPromise(onlyArgs,'template', () => run(mkdir('assets')))
    .then(_ => ifArgPromise(onlyArgs,'template', () => run(mkdir('html'))))
    .then(_ => ifArgPromise(onlyArgs,'template', () => run(mkdir('style'))))
    .then(_ => ifArgPromise(onlyArgs,'template', () =>  run(mkdir('script'))))
    .then(_ => {
      ifArg(onlyArgs,'template', () => fs.writeFileSync(process.cwd()+'/'+'script/main.js',''))
      ifArg(onlyArgs,'template', () => fs.writeFileSync(process.cwd()+'/'+'style/main.css',''))
      downloadFile(envFileUrl,'.env')
      ifArg(onlyArgs,'template', () => downloadFile(indexFileUrl,'html/index.html'))

      ifArgPromise(onlyArgs,'eslint', () => downloadFile(esLintUrl,'.eslintrc.js'))
      ifArgPromise(onlyArgs,'test', () => appendTestToProject())

      if(onlyArgs.indexOf('babel') > -1){

        if(onlyArgs.indexOf('template') > -1){
          downloadFile(appBabelTemplateFileUrl,'app.js')
        }else{
          downloadFile(appBabelFileUrl,'app.js')
        }
        downloadFile(runFileUrl,'run.js')
      }else{
        if(onlyArgs.indexOf('template') > -1){
          downloadFile(appFileUrl,'app.js')
        }else{
          downloadFile(appSimpleFileUrl, 'app.js')
        }
      }

      ifArgPromise(onlyArgs,'mysql', () => downloadFile(dbFileUrl,'db.js'))
      console.log('init complete');
    }).catch(console.log)
  })

.command('setup')
  .action(function(...args){
    const onlyArgs = transformBabel(args.filter(a => typeof a === 'string'))

    if(onlyArgs.indexOf('all') > -1){
      onlyArgs.push('mysql')
      onlyArgs.push('eslint')
      onlyArgs.push('template')
    }

    if(onlyArgs.indexOf('babel') > -1){
      if(!fs.existsSync(process.cwd()+'/'+'package.json')){
        console.log('no package.json')
        return
      }
      writeJsonFile(appendBabel(getJsonFile()))
    }

    ifArgPromise(onlyArgs,'template', () => run(mkdir('assets')))
    ifArgPromise(onlyArgs,'test', () => appendTestToProject())
    .then(_ => ifArgPromise(onlyArgs,'template', () => run(mkdir('html'))))
    .then(_ => ifArgPromise(onlyArgs,'template', () => run(mkdir('style'))))
    .then(_ => ifArgPromise(onlyArgs,'template', () =>  run(mkdir('script'))))
    .then(_ => {
      ifArg(onlyArgs,'template', () => fs.writeFileSync(process.cwd()+'/'+'script/main.js',''))
      ifArg(onlyArgs,'template', () => fs.writeFileSync(process.cwd()+'/'+'style/main.css',''))
      downloadFile(envFileUrl,'.env')
      ifArg(onlyArgs,'template', () => downloadFile(indexFileUrl,'html/index.html'))

       ifArg(onlyArgs,'eslint', () => downloadFile(esLintUrl,'.eslintrc.js'))

      if(onlyArgs.indexOf('babel') > -1){

        if(onlyArgs.indexOf('template') > -1){
          downloadFile(appBabelTemplateFileUrl,'app.js')
        }else{
          downloadFile(appBabelFileUrl,'app.js')
        }
        downloadFile(runFileUrl,'run.js')
      }else{
        downloadFile(appFileUrl,'app.js')
      }

      ifArgPromise(onlyArgs,'mysql', () => downloadFile(dbFileUrl,'db.js'))
      console.log('init complete');
    }).catch(console.log)
  })

program
  .command('init-babel')
  .action(function(env, options){
    if(!fs.existsSync(process.cwd()+'/'+'package.json')){
      console.log('no package.json')
      return
    }
    writeJsonFile(appendBabel(getJsonFile()))
    run(mkdir('assets'))
    .then(_ => run(mkdir('html')))
    .then(_ => run(mkdir('style')))
    .then(_ => run(mkdir('script')))
    .then(_ => {
      fs.writeFileSync(process.cwd()+'/'+'script/main.js','')
      fs.writeFileSync(process.cwd()+'/'+'style/main.css','')
      downloadFile(envFileUrl,'.env')
      downloadFile(indexFileUrl,'html/index.html')
      downloadFile(appBabelFileUrl,'app.js')
      downloadFile(runFileUrl,'run.js')
      downloadFile(dbFileUrl,'db.js')
      console.log('init complete');
    }).catch(console.log)
  })

program
  .command('setup-babel')
  .action(function(env, options){
    if(!fs.existsSync(process.cwd()+'/'+'package.json')){
      console.log('no package.json')
      return
    }
    writeJsonFile(appendBabel(getJsonFile()))
    run(mkdir('assets'))
    .then(_ => run(mkdir('html')))
    .then(_ => run(mkdir('style')))
    .then(_ => run(mkdir('script')))
    .then(_ => {
      fs.writeFileSync(process.cwd()+'/'+'script/main.js','')
      fs.writeFileSync(process.cwd()+'/'+'style/main.css','')
      downloadFile(envFileUrl,'.env')
      downloadFile(indexFileUrl,'html/index.html')
      downloadFile(appBabelFileUrl,'app.js')
      downloadFile(runFileUrl,'run.js')
      downloadFile(dbFileUrl,'db.js')
      console.log('init complete');
    }).catch(console.log)
  })

program.command('env')
  .action(function(env, options){
    downloadFile(envFileUrl,'.env')
  })

program.command('db')
  .action(function(env, options){
    downloadFile(dbFileUrl,'db.js')
  })

program.command('mysql')
  .action(function(env, options){
    downloadFile(dbFileUrl,'migration.js')
  })

program.command('migration')
  .action(function(env, options){
    downloadFile(dbFileUrl,'migration.js')
  })

program.command('upload')
  .action(function(env, options){
    uploadFiles(process.cwd())
  })

program.command('online')
  .action(function(env, options){
    uploadFiles(process.cwd())
  })

program.command('test')
  .action(function(env, options){
    appendTestToProject()
  })

program.command('pub')
  .action(function(env, options){
    uploadFiles(process.cwd())
  })

program.command('publish')
  .action(function(env, options){
    uploadFiles(process.cwd())
  })

program.parse(process.argv)
